# This Build.PL for Mojolicious-Static-Role-Compressed was generated by Dist::Zilla::Plugin::ModuleBuildTiny 0.015.
use strict;
use warnings;

use 5.012000;
use Module::Build::Tiny 0.034;
Build_PL();
